package net.rumati.logging.muffero.config.prop;

import java.io.File;
import java.io.FileNotFoundException;
import java.nio.charset.Charset;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.UnsupportedCharsetException;
import java.util.Map;
import net.rumati.logging.muffero.layout.Layout;
import net.rumati.logging.muffero.appender.ConsoleAppender;
import net.rumati.logging.muffero.appender.DatedFileAppender;
import net.rumati.logging.muffero.appender.FileAppender;
import net.rumati.logging.muffero.LoggingException;
import net.rumati.logging.muffero.Priority;
import net.rumati.logging.muffero.appender.RollingDatedFileAppender;
import net.rumati.logging.muffero.filter.NDCFilter;
import net.rumati.logging.muffero.filter.PriorityFilter;
import net.rumati.logging.muffero.layout.PatternLayout;

class DefaultPropertyObjectLoader
        extends PropertyObjectLoaderAggregator
{
    public DefaultPropertyObjectLoader()
    {
        addLoader(new SimplePropertyObjectLoader<PatternLayout>(PatternLayout.class)
        {
            @Override
            public PatternLayout load(Map<String, String> configuration, PropertyConfigContext ctx)
                    throws LoggingException
            {
                String pattern = configuration.get("pattern");
                if (pattern == null){
                    throw new LoggingException("No pattern provided for Layout: " + PatternLayout.class.getName());
                }
                return new PatternLayout(pattern);
            }
        });

        addLoader(new SimplePropertyObjectLoader<ConsoleAppender>(ConsoleAppender.class)
        {
            @Override
            public ConsoleAppender load(Map<String, String> configuration, PropertyConfigContext ctx)
                    throws LoggingException
            {
                String layoutClassName = configuration.get("layout");

                if (layoutClassName == null){
                    throw new LoggingException("No layout class provided for ConsoleAppender");
                }

                Layout layout = ctx.loadLayout(layoutClassName, "layout.", configuration);

                return new ConsoleAppender(layout);
            }
        });

        addLoader(new SimplePropertyObjectLoader<FileAppender>(FileAppender.class)
        {
            @Override
            public FileAppender load(Map<String, String> configuration, PropertyConfigContext ctx)
                    throws LoggingException
            {
                String fileName = configuration.get("file");
                if (fileName == null){
                    throw new LoggingException("No file name configured for " + FileAppender.class.getName());
                }

                String layoutClass = configuration.get("layout");
                if (layoutClass == null){
                    throw new LoggingException("No layout configured for " + FileAppender.class.getName());
                }
                Layout layout = ctx.loadLayout(layoutClass, "layout.", configuration);

                String encoding = configuration.get("encoding");
                try{
                    if (encoding != null){
                        Charset cs = null;
                        try{
                            cs = Charset.forName(encoding);
                        }catch (IllegalCharsetNameException e){
                            throw new LoggingException("Invalid encoding for " + FileAppender.class.getName(), e);
                        }catch (UnsupportedCharsetException e){
                            throw new LoggingException("Invalid encoding for " + FileAppender.class.getName(), e);
                        }

                        return new FileAppender(new File(fileName), cs, layout);
                    }else{
                        return new FileAppender(new File(fileName), layout);
                    }
                }catch (FileNotFoundException e){
                    throw new LoggingException("Invalid file for " + FileAppender.class.getName(), e);
                }
            }
        });

        addLoader(new SimplePropertyObjectLoader<DatedFileAppender>(DatedFileAppender.class)
        {
            @Override
            public DatedFileAppender load(Map<String, String> configuration, PropertyConfigContext ctx)
                    throws LoggingException
            {
                String fileNamePattern = configuration.get("fileNamePattern");
                if (fileNamePattern == null){
                    throw new LoggingException("No file name pattern configured for " + DatedFileAppender.class.getName());
                }

                String layoutClass = configuration.get("layout");
                if (layoutClass == null){
                    throw new LoggingException("No layout configured for " + DatedFileAppender.class.getName());
                }
                Layout layout = ctx.loadLayout(layoutClass, "layout.", configuration);

                Charset cs = null;
                String encoding = configuration.get("encoding");
                if (encoding != null){
                    cs = null;
                    try{
                        cs = Charset.forName(encoding);
                    }catch (IllegalCharsetNameException e){
                        throw new LoggingException("Invalid encoding for " + DatedFileAppender.class.getName(), e);
                    }catch (UnsupportedCharsetException e){
                        throw new LoggingException("Invalid encoding for " + DatedFileAppender.class.getName(), e);
                    }
                }

                try{
                    if (cs == null){
                        return new DatedFileAppender(fileNamePattern, layout);
                    }else{
                        return new DatedFileAppender(fileNamePattern, cs, layout);
                    }
                }catch (IllegalArgumentException e){
                    throw new LoggingException("Invalid fileNamePattern for " + DatedFileAppender.class.getName(), e);
                }
            }
        });

        addLoader(new SimplePropertyObjectLoader<RollingDatedFileAppender>(RollingDatedFileAppender.class)
        {
            @Override
            public RollingDatedFileAppender load(Map<String, String> configuration, PropertyConfigContext ctx)
                    throws LoggingException
            {
                String keepStr = configuration.get("keep");
                if (keepStr == null){
                    throw new LoggingException("No keep count configured for: " + RollingDatedFileAppender.class.getName());
                }

                int keep = 0;
                try {
                    keep = Integer.parseInt(keepStr);
                }catch(NumberFormatException e){
                    throw new LoggingException("Error parsing keep count: " + keepStr);
                }

                String fileNamePattern = configuration.get("fileNamePattern");
                if (fileNamePattern == null){
                    throw new LoggingException("No file name pattern configured for " + DatedFileAppender.class.getName());
                }

                String layoutClass = configuration.get("layout");
                if (layoutClass == null){
                    throw new LoggingException("No layout configured for " + DatedFileAppender.class.getName());
                }
                Layout layout = ctx.loadLayout(layoutClass, "layout.", configuration);

                Charset cs = null;
                String encoding = configuration.get("encoding");
                if (encoding != null){
                    cs = null;
                    try{
                        cs = Charset.forName(encoding);
                    }catch (IllegalCharsetNameException e){
                        throw new LoggingException("Invalid encoding for " + DatedFileAppender.class.getName(), e);
                    }catch (UnsupportedCharsetException e){
                        throw new LoggingException("Invalid encoding for " + DatedFileAppender.class.getName(), e);
                    }
                }

                try{
                    if (cs == null){
                        return new RollingDatedFileAppender(fileNamePattern, layout, keep);
                    }else{
                        return new RollingDatedFileAppender(fileNamePattern, cs, layout, keep);
                    }
                }catch (IllegalArgumentException e){
                    throw new LoggingException("Invalid fileNamePattern for " + DatedFileAppender.class.getName(), e);
                }
            }
        });

        addLoader(new SimplePropertyObjectLoader<PriorityFilter>(PriorityFilter.class)
        {
            @Override
            public PriorityFilter load(Map<String, String> configuration, PropertyConfigContext ctx)
                    throws LoggingException
            {
                String priorityName = configuration.get("priority");
                if (priorityName == null){
                    throw new LoggingException("No priority defined for: " + PriorityFilter.class.getName());
                }

                Priority p;
                try{
                    p = Priority.valueOf(priorityName);
                }catch (IllegalArgumentException e){
                    throw new LoggingException("Invalid priority: " + priorityName);
                }

                return new PriorityFilter(p);
            }
        });
        addLoader(new SimplePropertyObjectLoader<NDCFilter>(NDCFilter.class)
        {
            @Override
            public NDCFilter load(Map<String, String> configuration, PropertyConfigContext ctx)
                    throws LoggingException
            {
                String ndcKey = configuration.get("key");
                if (ndcKey == null){
                    throw new LoggingException("No NDC key defined for: " + NDCFilter.class.getName());
                }

                return new NDCFilter(ndcKey);
            }
        });
    }
}
