package net.rumati.logging.muffero.config.prop;

import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import net.rumati.logging.muffero.LoggingException;
import net.rumati.logging.muffero.appender.Appender;
import net.rumati.logging.muffero.filter.Filter;
import net.rumati.logging.muffero.layout.Layout;

public class PropertyConfigContext
{
    private final List<PropertyObjectLoader> loaders;
    private final Map<String, String> rootConfig;
    private final Map<String, Layout> namedLayouts = new TreeMap<String, Layout>();
    private final Map<String, Appender> namedAppenders = new TreeMap<String, Appender>();
    private final Map<String, Filter> namedFilters = new TreeMap<String, Filter>();

    PropertyConfigContext(List<PropertyObjectLoader> loaders, Map<String, String> config)
    {
        this.loaders = loaders;
        this.rootConfig = config;
    }

    private <T> T loadNameableObject(Class<T> objectType, String namedConfigPrefix, Map<String, T> cache, String name, String configPrefix, Map<String, String> config)
            throws LoggingException
    {
        if (!name.contains(".")){
            if (cache.containsKey(name)){
                return cache.get(name);
            }

            namedConfigPrefix += "." + name;

            if (rootConfig.containsKey(namedConfigPrefix)){
                T retv = loadObjectWithConfigPrefix(rootConfig.get(namedConfigPrefix), objectType, namedConfigPrefix + ".", rootConfig);

                cache.put(name, retv);

                return retv;
            }
        }
        return loadObjectWithConfigPrefix(name, objectType, configPrefix, config);
    }

    public Filter loadFilter(String name, String configPrefix, Map<String, String> config)
            throws LoggingException
    {
        return loadNameableObject(Filter.class, "muffero.filter", namedFilters, name, configPrefix, config);
    }

    public Appender loadAppender(String name, String configPrefix, Map<String, String> config)
            throws LoggingException
    {
        return loadNameableObject(Appender.class, "muffero.appender", namedAppenders, name, configPrefix, config);
    }

    public Layout loadLayout(String name, String configPrefix, Map<String, String> config)
            throws LoggingException
    {
        return loadNameableObject(Layout.class, "muffero.layout", namedLayouts, name, configPrefix, config);
    }

    public <T> T loadObjectWithConfigPrefix(String className, Class<T> typeClass, String configPrefix, Map<String, String> config)
            throws LoggingException
    {
        Map<String, String> resultConfig = new TreeMap<String, String>();
        for (Map.Entry<String, String> e : config.entrySet()){
            if (e.getKey().startsWith(configPrefix)){
                resultConfig.put(e.getKey().substring(configPrefix.length()), e.getValue());
            }
        }

        for (PropertyObjectLoader loader : loaders){
            if (loader.canLoad(className, typeClass)){
                return loader.loadObject(className, typeClass, resultConfig, this);
            }
        }

        try{
            Class c = Class.forName(className);
            if (!typeClass.isAssignableFrom(c)){
                throw new LoggingException("Requested class \"" + className + "\" does not implement \"" + typeClass.getName() + "\"");
            }
            @SuppressWarnings("unchecked")
            Class<? extends T> ctc = c;
            try{
                return ctc.newInstance();
            }catch (IllegalAccessException e){
                throw new LoggingException("Error instantiating object " + className, e);
            }catch (InstantiationException e){
                throw new LoggingException("Error instantiating object " + className, e);
            }catch (ExceptionInInitializerError e){
                throw new LoggingException("Error instantiating object " + className, e);
            }
        }catch (ClassNotFoundException e){
            throw new LoggingException(e);
        }
    }
}
