package net.rumati.logging.muffero.config.prop;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.TreeMap;
import net.rumati.logging.muffero.appender.Appender;
import net.rumati.logging.muffero.config.Configurator;
import net.rumati.logging.muffero.Logger;
import net.rumati.logging.muffero.LoggerFactory;
import net.rumati.logging.muffero.LoggingException;
import net.rumati.logging.muffero.filter.Filter;

public class PropertyConfigurator
        extends Configurator
{
    private final List<PropertyObjectLoader> loaders = new LinkedList<PropertyObjectLoader>();
    private final Map<String, String> config = new TreeMap<String, String>();

    public PropertyConfigurator(Properties properties)
    {
        this(propertiesToMap(properties));
    }

    private static Map<String, String> propertiesToMap(Properties properties)
    {
        Map<String, String> config = new TreeMap<String, String>();
        for (Map.Entry<Object, Object> e : properties.entrySet()){
            Object key = e.getKey();
            Object value = e.getValue();

            if (key != null && value != null){
                config.put(key.toString(), value.toString());
            }
        }
        return config;
    }

    public PropertyConfigurator(Map<String, String> config)
    {
        loaders.add(new DefaultPropertyObjectLoader());
        for (Map.Entry<String, String> e : config.entrySet()){
            this.config.put(e.getKey().trim(), e.getValue().trim());
        }
        for (Map.Entry<String, String> e : this.config.entrySet()){
            for (Map.Entry<String, String> e2 : this.config.entrySet()){
                if (!e2.getKey().equals(e.getKey())){
                    e2.setValue(e2.getValue().replace("${" + e.getKey() + "}", e.getValue()));
                }
            }
        }
    }

    public void addObjectLoader(PropertyObjectLoader loader)
    {
        loaders.add(loader);
    }

    public void configure()
            throws LoggingException
    {
        String loaderList = config.get("muffero.loaders");
        while (loaderList != null){
            String loaderClassName;
            int idx = loaderList.indexOf(",");
            if (idx < 0){
                loaderClassName = loaderList.trim();
                loaderList = null;
            }else{
                loaderClassName = loaderList.substring(0, idx).trim();
                loaderList = loaderList.substring(idx + 1).trim();
            }

            try{
                Class c = Class.forName(loaderClassName);
                if (!PropertyObjectLoader.class.isAssignableFrom(c)){
                    throw new LoggingException("Class \"" + c.getName() + "\" does not implement class \"" + PropertyObjectLoader.class.getName() + "\"");
                }
                try{
                    PropertyObjectLoader l = (PropertyObjectLoader)c.newInstance();
                    loaders.add(l);
                }catch (IllegalAccessException e){
                    throw new LoggingException("Error instantiating object loader " + loaderClassName, e);
                }catch (InstantiationException e){
                    throw new LoggingException("Error instantiating object loader " + loaderClassName, e);
                }catch (ExceptionInInitializerError e){
                    throw new LoggingException("Error instantiating object loader " + loaderClassName, e);
                }
            }catch (ClassNotFoundException e){
                throw new LoggingException("Unable to load object loader class: " + loaderClassName, e);
            }
        }

        PropertyConfigContext ctx = new PropertyConfigContext(loaders, config);

        for (Map.Entry<String, String> configEntry : config.entrySet()){
            String prefix = "muffero.logger.";
            String key = configEntry.getKey();
            if (key.startsWith(prefix) && key.length() > prefix.length()){
                String loggerName = key.substring(prefix.length());
                loadLoggerConfig(LoggerFactory.getLogger(loggerName), configEntry.getValue(), ctx);
            }
        }

        String rootLoggerConfig = config.get("muffero.rootLogger");
        if (rootLoggerConfig != null){
            loadLoggerConfig(LoggerFactory.getRootLogger(), rootLoggerConfig, ctx);
        }
    }

    private void loadLoggerConfig(Logger logger, String configuration, PropertyConfigContext ctx)
            throws LoggingException
    {
        while (configuration != null && configuration.length() > 0){
            String item;

            int commaIndex = configuration.indexOf(",");
            int paranIndex = configuration.indexOf("(");

            int index = commaIndex;
            if (index < 0 || (paranIndex >= 0 && paranIndex < index)){
                index = paranIndex;
            }

            if (index < 0){
                item = configuration.trim();
                configuration = null;
            }else{
                item = configuration.substring(0, index).trim();
                configuration = configuration.substring(index + ((index == commaIndex) ? 1 : 0)).trim();
            }

            if (item.equals("NOINHERIT")){
                logger.setInheritParentAppenders(false);
            }else{
                String appenderName = item;
                String[] filterNames = new String[0];

                if (configuration != null){
                    int closingParanIndex = configuration.indexOf(")");
                    if (configuration.startsWith("(") && closingParanIndex > 0){
                        filterNames = configuration.substring(1, closingParanIndex).split("\\,");
                        configuration = configuration.substring(closingParanIndex + 1).trim();
                        if (configuration.startsWith(",")){
                            configuration = configuration.substring(1);
                        }
                    }
                }
                Appender appender = ctx.loadAppender(appenderName, "INVALID", new TreeMap<String, String>());

                List<Filter> filters = new LinkedList<Filter>();
                for (String filterName : filterNames){
                    filterName = filterName.trim();
                    filters.add(ctx.loadFilter(filterName, "INVALID", new TreeMap<String, String>()));
                }

                if (config.containsKey("muffero.appender." + appenderName + ".filters")){
                    String filterListStr = config.get("muffero.appender." + appenderName + ".filters").trim();
                    if (filterListStr.length() > 0){
                        filterNames = filterListStr.split("\\,");
                        for (String filterName : filterNames){
                            filterName = filterName.trim();
                            filters.add(ctx.loadFilter(filterName, "INVALID", new TreeMap<String, String>()));
                        }
                    }
                }

                addAppender(logger, appender, filters);
            }
        }
    }
}
